----------------------------------------------------------------------------------------------------
--! @file gmii_to_mii.vhd
--! @brief GMII to MII wrapper for ZYNQ-7000 
--! @author Antti Lukats
--! @version 1.0
--! @date 2015
--! @copyright Copyright 2015 Trenz Electronic GmbH
--! @license BSD
--! @pre Vivado 2014.4+
----------------------------------------------------------------------------------------------------

--! Use standard library
library ieee;
--! Use logic elements
    use ieee.std_logic_1164.all;
 
entity gmii_to_mii is
    generic (
        C_MII_HAS_TXER  : boolean := false;          --! RXER present in MII (usually not used)
        C_USE_INT       : integer range 0 to 2 := 0; --! not implemented
        C_USE_PWRDOWN   : integer range 0 to 2 := 0; --! POWERDOWN pin use: 0 - not used, 1 - Constant High, 2 - TBD
        C_USE_RESET_N   : integer range 0 to 2 := 0  --! Reset pin use: 0 - not used, 1 - Constant High, 2 - bypass from PL Fabric
    );
    port ( 

    phy_resetn     : in std_logic := '1';
    phy_powerdown  : out std_logic;
    
    -- MII    
    mii_resetn     : out std_logic;     --! optional signal in MII, active low PHY Reset
    
    mii_txd        : out std_logic_vector(3 downto 0);
    mii_txclk      : in std_logic;
    mii_txen       : out std_logic;
    mii_txer       : out std_logic; --! optional signal usually not present
    
    mii_rxd        : in std_logic_vector(3 downto 0);
    mii_rxclk      : in std_logic;
    mii_rxdv       : in std_logic;
    mii_rxer       : in std_logic := '0';
    mii_col        : in std_logic := '0';
    mii_crs        : in std_logic := '0';
  
    -- GMII
    gmii_txd        : in std_logic_vector(7 downto 0);
    gmii_txclk      : out std_logic;
    gmii_txen       : in std_logic;
    gmii_txer       : in std_logic;
    
    gmii_rxd        : out std_logic_vector(7 downto 0);
    gmii_rxclk      : out std_logic;
    gmii_rxdv       : out std_logic;
    gmii_rxer       : out std_logic;
    
    gmii_col        : out std_logic;
    gmii_crs        : out std_logic
  );
end gmii_to_mii;
 
architecture RTL of gmii_to_mii is

begin
    mii_resetn <= phy_resetn;
    phy_powerdown <= '1';
    
    
    mii_txd(3 downto 0) <= gmii_txd(3 downto 0);
    mii_txen <= gmii_txen;
    mii_txer <= gmii_txer;

    gmii_rxd(7 downto 4) <= "0000"; -- tied down unused bits
    gmii_rxd(3 downto 0) <= mii_rxd(3 downto 0);

    gmii_rxdv <= mii_rxdv; 
    gmii_rxer <= mii_rxer; 
    
    gmii_col  <= mii_col;
    gmii_crs  <= mii_crs;
    
    gmii_rxclk <= mii_rxclk;
    gmii_txclk <= mii_txclk;


end RTL;